package com.winhc.repal.mq.producer;

import com.winhc.repal.mq.bean.SendMsgBean;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * 消息发送客户端
 * 
 * @author wowo
 *
 */
@Service("messageSendService")
public class MessageSendService {
    Log log = LogFactory.getLog(getClass());

    private LinkedBlockingQueue<SendMsgBean> messageRequestQueue = new LinkedBlockingQueue<>();

    @Resource
    private RepalProducer repalProducer;

    @PostConstruct
    public void init() {
        log.info("开始消息发送工作线程");
        new Thread(new Worker()).start();
    }

    /**
     * 发送消息
     * 
     * @param req
     */
    public void send(SendMsgBean req) {
        messageRequestQueue.offer(req);
    }

    /**
     * 工作线程， 处理短信发送和重发
     * 
     * @author wowo
     *
     */
    public class Worker implements Runnable {

        @Override
        public void run() {
            while (true) {
                SendMsgBean req = null;
                try {
                    req = messageRequestQueue.take();
                } catch (InterruptedException e) {
                    log.error("", e);
                }
                if (req == null) {
                    continue;
                }
                connect(req);
            }
        }

        private void connect(SendMsgBean req) {
            try {
                String res = repalProducer.sendMsg(req);
                log.info("消息发送结果：" + res);
            } catch (Exception e) {
                log.error("发送消息异常", e);
            }
        }
    }

}
