package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.repal.entity.RepalResponsiblePerson;
import com.winhc.repal.enums.DeletedStatusEnum;
import com.winhc.repal.model.vo.ResponsiblePersonNameVO;
import com.winhc.repal.repository.RepalResponsiblePersonMapper;
import com.winhc.repal.service.RepalResponsiblePersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @description RepalResponsiblePerson 接口实现类
 * @author Generator
 * @date 2022-04-08
 */
@Service
public class RepalResponsiblePersonServiceImpl extends ServiceImpl<RepalResponsiblePersonMapper, RepalResponsiblePerson> implements RepalResponsiblePersonService {

    @Autowired
    private RepalResponsiblePersonMapper repalResponsiblePersonMapper;

    @Override
    public Map<Long, List<String>> getResponsiblePersonNameList(List<Long> billIdList) {
        if (CollUtil.isEmpty(billIdList)) {
            return new HashMap<>();
        }
        List<ResponsiblePersonNameVO> nameList = repalResponsiblePersonMapper.getResponsiblePersonNameList(billIdList);
        if (CollUtil.isEmpty(nameList)) {
            return new HashMap<>();
        }
        return nameList.stream().collect(Collectors.groupingBy(ResponsiblePersonNameVO::getRepalBillId, Collectors.mapping(ResponsiblePersonNameVO::getUserName, Collectors.toList())));
    }

    @Override
    public List<String> getResponsiblePersonName(Long billId) {
        List<ResponsiblePersonNameVO> nameList = repalResponsiblePersonMapper.getResponsiblePersonNameList(Collections.singletonList(billId));
        if (CollUtil.isEmpty(nameList)) {
            return new ArrayList<>();
        }
        return nameList.stream().map(ResponsiblePersonNameVO::getUserName).collect(Collectors.toList());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deletePersonByBookId(Long repalBookId) {
        this.update(Wrappers.lambdaUpdate(RepalResponsiblePerson.class)
                .eq(RepalResponsiblePerson::getRepalBookId, repalBookId)
                .set(RepalResponsiblePerson::getDeleted, DeletedStatusEnum.DELETE.getCode()));
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deletePersonByBillId(Long repalBillId) {
        this.update(Wrappers.lambdaUpdate(RepalResponsiblePerson.class)
                .eq(RepalResponsiblePerson::getRepalBillId, repalBillId)
                .set(RepalResponsiblePerson::getDeleted, DeletedStatusEnum.DELETE.getCode()));
        return true;
    }
}
